@@ -1,38 +1,30 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdlib.h>
 #include <time.h>
 #include <sys/time.h>
 #include <cgic.h>
-#include <iconv.h>
 #include <errno.h>
+#include "alloc.h"
+#include "conv.h"
 #include "util.h"
 #include "hook.h"
 
-unsigned long url_encode(unsigned char*, unsigned char*, unsigned long);
+/* プロジェクトコード */
+char g_project_code[DEFAULT_LENGTH] = "";
+char g_project_code_4_url[DEFAULT_LENGTH] = "";
+/* アクション名 */
+char g_action_name[DEFAULT_LENGTH] = "";
+/* PATH_INFO */
+char g_path_info[DEFAULT_LENGTH] = "";
+
 static Action* get_actions();
 
+void redirect_raw(const char*);
+void page_not_found();
+
 Action* actions = NULL;
-int alloc_count = 0;
 
-void* xalloc(size_t size)
-{
-    void* p;
-    p = calloc(1, size);
-    if (!p) {
-        die("memory error.");
-    }
-    alloc_count++;
-/*     d("xalloc: %p\n", p); */
-    return p;
-}
-void xfree(void* p)
-{
-/*     d("xfree: %p\n", p); */
-    alloc_count--;
-    free(p);
-}
 static Action* get_actions()
 {
     return actions;
@@ -61,29 +53,101 @@ void free_action_actions()
         xfree(old);
     }
     xfree(actions);
-    d("alloc_count: %d\n", alloc_count);
+    d("alloc_count: %d\n", get_alloc_count());
 }
 
-void exec_action()
+/* 復帰できないエラーが発生した場合にエラーページを表示する。 */
+void print_error_page(char* file_name, int line_number, char* message)
+{
+    o("Status: 500 Starbug1 Internal Error.\r\n");
+    o("Content-Type: text/html\r\n\r\n");
+    o(  "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">"
+        "<html xmlns=\"http://www.w3.org/1999/xhtml\">"
+        "<head>"
+            "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\" />"
+        "</head>"
+        "<body>"
+            "<h2>ERROR oops!</h2>"
+            "エラーが発生しましたm(_ _)m"
+            "<h3>detail information</h3>");
+    o(      "<p>[%s(%d)] %s</p>",
+                file_name,
+                line_number,
+                message);
+    o(      "<hr />"
+            "<div align=\"right\">"
+            "<address>%s.</address>"
+            "</div>"
+        "</body>"
+        "</html>", COPYRIGHT);
+}
+
+ActionType analysis_action()
 {
     char* index;
+    char* script_name = cgiScriptName;
     char* path_info = cgiPathInfo;
-    char action_name[1024];
-    Action* a;
-    if (strlen(path_info) > 1) {
-        strncpy(action_name, path_info + 1, 1024);
-    } else {
-        strcpy(action_name, "");
-    }
-    if ((index = strchr(action_name, '/'))) {
-        *index = '\0';
-    }
+    char index_cgi_file_name[DEFAULT_LENGTH] = "";
+    char admin_cgi_file_name[DEFAULT_LENGTH] = "";
+    ActionType ret = ACTION_TYPE_NONE;
 
+    memset(g_project_code, 0, DEFAULT_LENGTH);
+    memset(g_action_name, 0, DEFAULT_LENGTH);
+    memset(g_path_info, 0, DEFAULT_LENGTH);
+    d("************************************\n");
+    d("REQUEST: %s%s\n", script_name, path_info);
+    if (strlen(path_info) > 1) {
+        strncpy(g_project_code, path_info + 1, DEFAULT_LENGTH);
+    } else {
+        redirect_raw("top");
+        return ret; /* ACTION_TYPE_NONE を返却する。 */
+    }
+    if ((index = strchr(g_project_code, '/'))) {
+        *index = '\0';
+        strncpy(g_action_name, index + 1, DEFAULT_LENGTH - (strlen(g_project_code) + 1));
+        if ((index = strchr(g_action_name, '/'))) {
+            *index = '\0';
+            /* 残りをpath_infoとする */
+            strcpy(g_path_info, index + 1);
+        }
+    }
+    /* URLとして使われるプロジェクトID。URL encodeする。 */
+    url_encode((unsigned char*)g_project_code, (unsigned char*)g_project_code_4_url, DEFAULT_LENGTH);
+
+    sprintf(index_cgi_file_name, "index.%s", get_ext(cgiScriptName));
+    sprintf(admin_cgi_file_name, "admin.%s", get_ext(cgiScriptName));
+    /* 実行するモードの判定 */
+    if (strstr(script_name, index_cgi_file_name)) {
+        if (strcmp(g_project_code, "top") == 0) {
+            d("actiontype top\n");
+            ret = ACTION_TYPE_INDEX_TOP;
+        } else {
+            d("actiontype project\n");
+            ret = ACTION_TYPE_INDEX;
+        }
+    } else if (strstr(script_name, admin_cgi_file_name)) {
+        d("g_project_code: %s\n", g_project_code);
+        if (strcmp(g_project_code, "top") == 0) {
+            d("actiontype admin top\n");
+            ret = ACTION_TYPE_ADMIN_TOP;
+        } else {
+            d("actiontype admin project\n");
+            ret = ACTION_TYPE_ADMIN;
+        }
+    }
+    return ret;
+}
+void exec_action()
+{
+    Action* a;
+    char action_name[DEFAULT_LENGTH];
+    strcpy(action_name, g_action_name);
     for (a = get_actions(); a != NULL; a = a->next) {
+/*         d("action: %s %s\n", a->action_name, action_name); */
         if (!strcmp(action_name, a->action_name)) {
-            d("exec_action start: %s\n", a->action_name);
+            d("=exec_action start: %s\n", a->action_name);
             a->action_func();
-            d("exec_action end  : %s\n", a->action_name);
+            d("=exec_action end  : %s\n", a->action_name);
             return;
         }
     }
@@ -93,8 +157,10 @@ void exec_action()
     } else {
         /* path_infoが空なら、top_actionを呼び出す。 */
         for (a = get_actions(); a != NULL; a = a->next) {
-            if (!strcmp("top", a->action_name)) {
+            if (!strcmp("top", a->action_name) || !strcmp("top_top", a->action_name)) {
+                d("=exec_action default_action start: %s\n", a->action_name);
                 a->action_func();
+                d("=exec_action default_action end  : %s\n", a->action_name);
                 return;
             }
         }
@@ -106,19 +172,78 @@ void exec_action()
             return cgiFormIO; \
         } \
     } 
-int get_ticket_syntax_len(char* data, size_t len)
+static const char PROJECT_SPLITTER = ':';
+
+int get_ticket_syntax_len(char* data, size_t len, char* project_id, char* ticket_id)
 {
-    int ticket_syntax_len = 0;
-    if (len == 0) return ticket_syntax_len; /* 最後の文字だった */
-    data += 1; /* 1文字進める。 */
-    while (len--) {
-        if (*data < '0' || *data > '9') 
-            break; /* 数字でない場合は、breakする。 */
-        data++;
-        ticket_syntax_len++;
+    char string[1024];
+    int index = 0;
+    char* block_p;
+    char* line_p;
+
+    memset(string, '\0', 1024);
+    strncpy(string, data, MIN(len, 1024 - 1));
+    if (len == 0) return index; /* 最後の文字だった */
+
+    block_p = strchr(string, ' ');
+    line_p = strchr(string, '\n');
+    if (!block_p)
+        block_p = string + strlen(string);
+    if (!line_p)
+        line_p = string + strlen(string);
+
+    /* 解析対象の文字列を特定 */
+    string[MIN(block_p - string, line_p - string)] = '\0';
+
+    {
+        char* project_mode = strchr(string, PROJECT_SPLITTER);
+        char* project_id_p = string + 1;
+        char* ticket_no_p = string + 1;
+
+        while (index++, 1) { /* each char */
+            /* 
+             * 一文字つづ進めていく。
+             * 途中でチケットリンクでないと判定された場合は、index を 0 にして、breakする。
+             * チケットリンクが終了したら、breakする。
+             */
+            if (string[index] == '\0') {
+                break;
+            } else if (project_mode) {
+                /* #プロジェクト:チケットモード */
+                if (string[index] == PROJECT_SPLITTER) {
+                    /* プロジェクトIDモードであり、:を検出した場合 */
+                    ticket_no_p = &string[index + 1];
+                    /* :で終了している場合は、チケットリンクではない。 */
+                    if (string[index + 1] < '0' || string[index + 1] > '9') {
+                        index = 0; break;
+                    }
+                } else if (project_id_p != ticket_no_p && (string[index] < '0' || string[index] > '9')) {
+                    /* チケットNO検索中に、数字以外を検出した場合 */
+                    string[index] = '\0'; break; /* 一つ前までがチケットリンクとなる。 */
+                }
+            } else if (!project_mode) {
+                /* #チケットモード */
+                if (string[index] >= '0' && string[index] <= '9') {
+                    /* プロジェクトIDモードではなく、数字を検出した場合 (正常) */
+                } else {
+                    /* reached to end. */
+                    string[index] = '\0'; break; /* 一つ前までがチケットリンクとなる。 */
+                }
+            }
+        }
+        if (index < 2) {
+            /* チケットリンクではない。 */
+            return 0;
+        }
+        if (project_id_p != ticket_no_p) {
+            strncpy(project_id, project_id_p, (ticket_no_p - 1) - project_id_p);
+        }
+        strcpy(ticket_id, ticket_no_p);
+        /*         index = (ticket_no_p - string) + strlen(ticket_no_p); */
     }
-    return ticket_syntax_len;
+    return index;
 }
+
 /*
  * 複数行テキストの領域では、pre記法をサポートする。
  * チケットリンクをサポートする。
@@ -129,16 +254,30 @@ static cgiFormResultType cgiHtmlEscapeDataMultiLine(char *data, int len)
     while (len--) {
         if (*data == '#') {
             int ticket_syntax_len;
-            ticket_syntax_len = get_ticket_syntax_len(data, len);
+            char project_id[DEFAULT_LENGTH];
+            char ticket_id[DEFAULT_LENGTH];
+            memset(project_id, '\0', DEFAULT_LENGTH);
+            memset(ticket_id, '\0', DEFAULT_LENGTH);
+            /* ループの最初でlenが減っているので、get_ticket_syntax_lenに渡すときは戻して呼び出す。 */
+            ticket_syntax_len = get_ticket_syntax_len(data, len + 1, project_id, ticket_id);
+            d("ticket_syntax_len: %d \n", ticket_syntax_len);
             if (ticket_syntax_len == 0) {
                 TRYPUTC(*data);
             } else {
-                char ticket_id[DEFAULT_LENGTH];
-                strncpy(ticket_id, ++data, ticket_syntax_len);
-                ticket_id[ticket_syntax_len] = '\0';
-                o("<a href=\"%s/ticket/%s\">#%s</a>", cgiScriptName, ticket_id, ticket_id);
+                char display_text[DEFAULT_LENGTH];
+                if (strlen(project_id) == 0) {
+                    strcpy(project_id, g_project_code_4_url);
+                    strcpy(display_text, ticket_id);
+                } else {
+                    sprintf(display_text, "%s:%s", project_id, ticket_id);
+                }
+                o("<a href=\"%s/%s/ticket/%s\">#%s</a>",
+                        cgiScriptName, 
+                        project_id, 
+                        ticket_id, 
+                        display_text);
                 data += ticket_syntax_len - 1;
-                len -= ticket_syntax_len;
+                len -= ticket_syntax_len - 1;
             }
         } else if (len > 2 && *data == '>' &&
                 *(data + 1) == '|' &&
@@ -233,6 +372,49 @@ static cgiFormResultType cgiHtmlEscapeDataMailaddress(char *data, int len)
     }
     return cgiFormSuccess;
 }
+static cgiFormResultType cgi_html_escape(char *data, int len)
+{
+    while (len--) {
+        if (*data == '<') {
+            TRYPUTC('&');
+            TRYPUTC('l');
+            TRYPUTC('t');
+            TRYPUTC(';');
+        } else if (*data == '&') {
+            TRYPUTC('&');
+            TRYPUTC('a');
+            TRYPUTC('m');
+            TRYPUTC('p');
+            TRYPUTC(';');
+        } else if (*data == '>') {
+            TRYPUTC('&');
+            TRYPUTC('g');
+            TRYPUTC('t');
+            TRYPUTC(';');
+        } else if (*data == '"') {
+            TRYPUTC('&');
+            TRYPUTC('q');
+            TRYPUTC('u');
+            TRYPUTC('o');
+            TRYPUTC('t');
+            TRYPUTC(';');
+        } else if (*data == '\'') {
+            TRYPUTC('&');
+            TRYPUTC('#');
+            TRYPUTC('3');
+            TRYPUTC('9');
+            TRYPUTC(';');
+        } else {
+            TRYPUTC(*data);
+        }
+        data++;
+    }
+    return cgiFormSuccess;
+}
+void cgi_escape(char *s)
+{
+    cgi_html_escape(s, (int) strlen(s));
+}
 
 void hmail(char *s)
 {
@@ -245,55 +427,6 @@ void u(char* str)
     url_encode((unsigned char*)str, (unsigned char*)buf, DEFAULT_LENGTH);
     fprintf(cgiOut, "%s", buf);
 }
-unsigned long url_encode(unsigned char* csource, unsigned char* cbuffer, unsigned long lbuffersize)
-{
-    unsigned long llength;
-    unsigned long lcount = 0;
-    unsigned char cbyte;
-    unsigned char ctemp[4];
-    unsigned long lresultcount = 0;
-
-    llength = (unsigned long)strlen((char*)csource);
-    if(!llength) { return lresultcount; }
-    if(lbuffersize < (llength * 3 + 1)) { return lresultcount; }
-
-    while(1) {
-        cbyte = *(csource + lcount);
-        if( ((cbyte >= 0x81) && (cbyte <= 0x9F)) ||
-                ((cbyte >= 0xE0) && (cbyte <= 0xEF)) ) {            /* Shift-JIS 2 バイト文字だった場合 */
-            sprintf((char*)ctemp, "%%%02X", cbyte);                        /* URL エンコード(上位バイト) */
-            strncpy((char*)(cbuffer + lresultcount), (char*)ctemp, 4);
-            lcount++;
-            lresultcount += 3;
-            if(lcount == llength) { break; }                        /* 文字列の終端に達した場合、ループを抜ける */
-            sprintf((char*)ctemp, "%%%02X", *(csource + lcount));          /* URL エンコード(下位バイト) */
-            strncpy((char*)cbuffer + lresultcount, (char*)ctemp, 4);
-            lcount++;
-            lresultcount += 3;
-        } else if(cbyte == 0x20) {                                  /* 1 バイト半角スペース(" ")だった場合 */
-            strncpy((char*)cbuffer + lresultcount, "+", 2);
-            lcount++;
-            lresultcount++;
-        } else if( ((cbyte >= 0x40) && (cbyte <= 0x5A)) ||          /* @A-Z */
-                ((cbyte >= 0x61) && (cbyte <= 0x7A)) ||             /* a-z */
-                ((cbyte >= 0x30) && (cbyte <= 0x39)) ||             /* 0-9 */
-                (cbyte == 0x2A) ||                                  /* "*" */
-                (cbyte == 0x2D) ||                                  /* "-" */
-                (cbyte == 0x2E) ||                                  /* "." */
-                (cbyte == 0x5F) ) {                                 /* "_" */ /* 無変換文字だった場合 */
-            strncpy((char*)cbuffer + lresultcount, (char*)csource + lcount, 2);
-            lcount++;
-            lresultcount++;
-        } else {                                                    /* その他の文字の場合 */
-            sprintf((char*)ctemp, "%%%02X", cbyte);                        /* URL エンコード */
-            strncpy((char*)cbuffer + lresultcount, (char*)ctemp, 4);
-            lcount++;
-            lresultcount += 3;
-        }
-        if(lcount == llength) { break; }
-    }
-    return lresultcount;                                            /* cbuffer に書き込んだ文字列のサイズを返す */
-}
 char* get_filename_without_path(char* path)
 {
     char* p = path;
@@ -359,43 +492,18 @@ ElementFile* get_upload_content(const int element_id)
     return content;
 }
 
-static unsigned char *base64 = (unsigned char *)"ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
-
-static void enclode_char(unsigned long bb, int srclen, unsigned char *dist, int j)
+void page_not_found()
 {
-    int x, i, base;
-
-    for (i = srclen; i < 2; i++) 
-        bb <<= 8;
-    for (base = 18, x = 0; x < srclen + 2; x++, base -= 6) {
-        dist[j++] = base64[(unsigned long)((bb>>base) & 0x3F)];
-    }
-    for (i = x; i < 4; i++) {
-        dist[j++] = (unsigned char)'=';
-    }
+    o("Status: 404 Not Found.\r\n");
+    o("Content-Type: text/plain; charset=utf-8;\r\n\r\n");
+    o("404 Not Found.\r\n");
 }
-
-void base64_encode(const unsigned char *src, unsigned char *dist)
+void redirect_raw(const char* path)
 {
-    unsigned char *p = (unsigned char*)src;
-    unsigned long bb = (unsigned long)0;
-    int i = 0, j = 0;
-
-    while (*p) {
-        bb <<= 8;
-        bb |= (unsigned long)*p;
-
-        if (i == 2) {
-            enclode_char(bb, i, dist, j);
-            j = j + 4;
-            i = 0;
-            bb = 0;
-        } else {
-            i++;
-        }
-        p++;
-    }
-    if (i) enclode_char(bb, i - 1, dist, j);
+    char redirecturi[DEFAULT_LENGTH];
+    sprintf(redirecturi, "%s/%s", cgiScriptName, path);
+    o("Status: 302 Temporary Redirection\r\n");
+    cgiHeaderLocation(redirecturi);
 }
 void redirect(const char* path, const char* message)
 {
@@ -411,7 +519,7 @@ void redirect(const char* path, const char* message)
         strcat(uri, "?message=");
         strcat(uri, parambuf);
     }
-    sprintf(redirecturi, "%s%s", cgiScriptName, uri);
+    sprintf(redirecturi, "%s/%s%s", cgiScriptName, g_project_code, uri);
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
@@ -445,7 +553,7 @@ void redirect_with_hook_messages(const char* path, const char* message, List* re
             i++;
         }
     }
-    sprintf(redirecturi, "%s%s", cgiScriptName, uri);
+    sprintf(redirecturi, "%s/%s%s", cgiScriptName, g_project_code, uri);
     o("Status: 302 Temporary Redirection\r\n");
     cgiHeaderLocation(redirecturi);
 }
@@ -464,27 +572,9 @@ static cgiFormResultType csv_escape(char *data, int len)
 }
 void csv_field(char* src)
 {
-    iconv_t ic;
-    size_t src_size, dist_size, ret_len;
-    char* dist_a;
-    char* dist_p;
-    bool has_error = false;
-
-    src_size = strlen(src) + 1;
-    dist_size = src_size;  /* UTF-8からCP932なので、長さが短かくなることはない。そのためdist_sizeにも同じ長さを指定する。*/
-    dist_p = dist_a = xalloc(sizeof(char) * dist_size);
-    /* 文字コード変換処理 *          */
-    ic = iconv_open("CP932", "UTF-8");
-    ret_len = iconv(ic, &src, &src_size, &dist_p, &dist_size);
-    if (ret_len == (size_t)-1) {
-        d("iconv failed: %s", strerror(errno));
-        has_error = true;
-    }
     o("\"");
-    if (!has_error) csv_escape(dist_a, strlen(dist_a));
+    csv_escape(src, strlen(src));
     o("\"");
-    iconv_close(ic);
-    xfree(dist_a);
 }
 static cgiFormResultType cgiCssClassName(char *data, int len)
 {
@@ -517,14 +607,16 @@ void css_field(char* str)
 }
 String* get_base_url(String* buf)
 {
-    int is_ssl = strcmp(cgiServerProtocol, "HTTPS") == 0 ? 1 : 0;
+    char* https = getenv("HTTPS");
+    bool is_ssl = (https == '\0') ? false : true;
+    int server_port = atoi(cgiServerPort);
     int is_default_port_no = 
-        (is_ssl && strcmp(cgiServerPort, "443") == 0) || (is_ssl == 0 && strcmp(cgiServerPort, "80") == 0);
+        (is_ssl && server_port == 443) || (is_ssl == 0 && server_port == 80);
     string_appendf(buf, "http%s://%s%s%s%s",
             is_ssl ? "s" : "",
+            cgiServerName,
             is_default_port_no ? "" : ":",
             is_default_port_no ? "" : cgiServerPort,
-            cgiServerName,
             cgiScriptName);
     return buf;
 }
@@ -558,82 +650,7 @@ void get_cookie_string(char* key, char* buf)
     base64_decode((unsigned char*)value, (unsigned char*)value_base64);
     strcpy(buf, value_base64);
 }
-typedef union {
-    unsigned int    data;
-    unsigned char   xyz[4];
-} Union_data_t;
 
-static long codetovalue(unsigned char c)
-{
-    if( (c >= (unsigned char)'A') && (c <= (unsigned char)'Z') ) {
-        return (long)(c - (unsigned char)'A');
-    }
-    else if( (c >= (unsigned char)'a') && (c <= (unsigned char)'z') ) {
-        return ((long)(c - (unsigned char)'a') +26);
-    }
-    else if( (c >= (unsigned char)'0') && (c <= (unsigned char)'9') ) {
-        return ((long)(c - (unsigned char)'0') +52);
-    }
-    else if( (unsigned char)'+' == c ) {
-        return (long)62;
-    }
-    else if( (unsigned char)'/' == c ) {
-        return (long)63;
-    }
-    else if( (unsigned char)'=' == c ) {
-        return (long)0;
-    }
-    else {
-        return -1;
-    }
-}
-
-static int decode_str(int enc_ptr, const unsigned char *src, unsigned char *dest)
-{
-    int i, j;
-    unsigned long base64 = 0;
-    unsigned char x;
-    Union_data_t bb;
-
-    for (i = enc_ptr; i < enc_ptr + 4; i++) {
-        if (src[i] == '\0') return -1; /* '\0'が出現したら終了。 */
-        x = codetovalue(src[i]);
-        base64 |= x;
-        if ((i - enc_ptr) != 3) base64 <<= 6;
-    }
-    base64 <<= 8;
-    bb.data = base64;
-
-    for(j = 0, i = 3; i >= 1; i--) {
-        dest[j++] = bb.xyz[i];
-    }
-    return 0;
-}
-
-int base64_decode(const unsigned char *src, unsigned char *dest)
-{
-    int i = 0;
-    int srclen;
-    char tmp[4];
-
-    /* NULL pointer */
-    if (src == NULL) return -1;
-
-    srclen = strlen((char*)src);
-
-    if (!srclen % 4) return -2;
-
-    while(srclen) {
-        memset(tmp, 0, sizeof(tmp));
-        if (decode_str(i, src, (unsigned char*)tmp) != 0)
-            return -1; /* 失敗した場合は、終了 */
-        strcat((char*)dest, tmp);
-        i += 4;
-        srclen -= 4;
-    }
-
-    return 0;
-}
 void set_date_string(char* buf)
 {
     struct timeval tv;
@@ -663,4 +680,39 @@ char* get_ext(char* path)
     }
     return "";
 }
+void print_field_help()
+{
+    o(      "<div class=\"description\">\n"
+            "\t<ul>\n"
+            "\t\t<li>%s</li>\n"
+            "\t\t<li>%s</li>\n"
+            "\t\t<li>%s</li>\n"
+            "\t</ul>\n"
+            "</div>\n",
+            _("[ticket form syntax description1]"),
+            _("[ticket form syntax description2]"),
+            _("[ticket form syntax description3]"));
+}
+void set_locale(char* locale)
+{
+    char locale_utf8[DEFAULT_LENGTH];
+    sprintf(locale_utf8, "%s.UTF-8", locale);
+    d("locale: %s\n", locale_utf8);
+#ifndef _WIN32
+    setenv("LANG", locale_utf8, 1); /* FreeBSD(さくらインターネット)でsetlocaleが動作しない場合があったため、環境変数を書き換える。 */
+#else
+    {
+        char envstr[256];
+        strcpy(envstr, "LANG=");
+        strcat(envstr, locale_utf8);
+        putenv(envstr);
+    }
+#endif
+    d("setlocale: %s\n", setlocale(LC_ALL, locale_utf8));
+    d("bindtextdomain: %s\n", bindtextdomain("starbug1", "locale"));
+    d("textdomain: %s\n", textdomain("starbug1"));
+#ifdef _WIN32
+    bind_textdomain_codeset("starbug1", "utf-8");
+#endif
+}
 /* vim: set ts=4 sw=4 sts=4 expandtab fenc=utf-8: */
